
/*
 * $Id: opts.cc,v 1.1.1.1 1999/01/27 08:40:44 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "Assert.h"
#include "LogComment.h"
#include "UniqId.h"
#include "opts.h"
#include "gadgets.h"
#include "globals.h"

/* opts tools */


static const char *valStr(const char *val) { return val ? val : "<none>"; }

static
bool splitVal(const char *val, String &pref, String &suf) {
	if (val) {
		pref = val;
		if (char *h = strchr(pref, ':')) {
			*h = '\0';
			suf = h+1;
			return true;
		}
	}
	//cerr << ThePrgName << ": expected value with a column (':'), got '" << valStr(val) << "'" << endl;
	return false;
}


/* Bool */

BoolOpt::BoolOpt(OptGrp *aGrp, const char *aName, const char *aDescr, bool def = false):
	Opt(aGrp, aName, aDescr), theVal(def) {
}

bool BoolOpt::parse(const char *name, const char *val) {
	if (val) {
		if (!strcmp(val, "1") || !strcmp(val, "yes") || !strcmp(val, "y") || !strcmp(val, "on")) {
			theVal = true;
			return true;
		} else
		if (!strcmp(val, "0") || !strcmp(val, "no") || !strcmp(val, "n") || !strcmp(val, "off")) {
			theVal = false;
			return true;
		}
	}
	cerr << ThePrgName << ": boolean (1|0|yes|no|on|off) value expected for the `" << name << "` option; got `" << valStr(val) << "'" << endl;
	return false;
}

void BoolOpt::report() const {
	Comment << (theName[0] == 'd' ? (theVal ? "yes" : "no") : (theVal ? "on" : "off"));
}

/* Int */

IntOpt::IntOpt(OptGrp *aGrp, const char *aName, const char *aDescr, int def = -1):
	Opt(aGrp, aName, aDescr), theVal(def) {
}

bool IntOpt::parse(const char *name, const char *val) {
	if (val) {
		if (!strcmp(val, "none")) {
			theVal = -1;
			return true;
		} else
		if (!strcmp(val, "max")) {
			theVal = 0x7fffffff;
			return true;
		} else {
			char *ptr = 0;
			const int h = (int) strtol(val, &ptr, 10);
			if (ptr != val && ptr && !*ptr) {
				theVal = h;
				return true;
			}
		}
	}
	
	cerr << ThePrgName << ": integer value expected for the `" << name << "` option; got `" << valStr(val) << "'" << endl;
	return false;
}

void IntOpt::report() const {
	Comment << theVal;
}


/* Time */

TimeOpt::TimeOpt(OptGrp *aGrp, const char *aName, const char *aDescr, Time def):
	Opt(aGrp, aName, aDescr), Time(def), theVal(*this) {
}

bool TimeOpt::parse(const char *name, const char *val) {
	if (val) {
		if (!strcmp(val, "none")) {
			theVal = Time();
			return true;
		} else
		if (!strcmp(val, "inf")) {
			theVal = Time(0x7fffffff, 0);
			return true;
		} else {
			char *ptr = 0;
			const int h = (int) strtol(val, &ptr, 10);

			if (ptr != val && ptr) {
				if (!strcmp(ptr, "msec") || !strcmp(ptr, "ms")) {
					theVal = Time::Msec(h);
					return true;
				} else
				if (!strcmp(ptr, "sec") || !strcmp(ptr, "s")) {
					theVal = Time::Sec(h);
					return true;
				} else
				if (!strcmp(ptr, "min")) {
					theVal = Time::Sec(h*60);
					return true;
				} else
				if (!strcmp(ptr, "hr")) {
					theVal = Time::Sec(h*60*60);
					return true;
				}
			}
		}
	}
	cerr << ThePrgName << ": time value (e.g., '30msec') expected for the `" << name << "` option; got `" << valStr(val) << "'" << endl;
	return false;

}

void TimeOpt::report() const {
	Comment << theVal;
}


/* Size */

SizeOpt::SizeOpt(OptGrp *aGrp, const char *aName, const char *aDescr, Size def):
	Opt(aGrp, aName, aDescr), Size(def), theVal(*this) {
}

bool SizeOpt::parse(const char *name, const char *val) {
	if (val) {
		if (!strcmp(val, "none")) {
			theVal = Size();
			return true;
		} else
		if (!strcmp(val, "inf")) {
			theVal = 0x7fffffff;
			return true;
		} else {
			char *ptr = 0;
			const int h = (int) strtol(val, &ptr, 10);

			if (ptr != val && ptr) {
				if (!strcmp(ptr, "byte") || !strcmp(ptr, "bytes") || !strcmp(ptr, "B")) {
					theVal = h;
					return true;
				} else
				if (!strcmp(ptr, "KB") || !strcmp(ptr, "K")) {
					theVal = Size::KB(h);
					return true;
				} else
				if (!strcmp(ptr, "MB")) {
					theVal = Size::MB(h);
					return true;
				}
			}
		}
	}
	cerr << ThePrgName << ": size value (e.g., '10KB') expected for the `" << name << "` option; got `" << valStr(val) << "'" << endl;
	return false;

}

void SizeOpt::report() const {
	Comment << theVal;
}


/* String */

StrOpt::StrOpt(OptGrp *aGrp, const char *aName, const char *aDescr, const char *def = 0):
	Opt(aGrp, aName, aDescr), theVal(def) {
}

bool StrOpt::parse(const char *name, const char *val) {
	if (val) {
		if (!strcmp(val, "unique"))
			theVal = Id2Str(UniqId::Create());
		else
			theVal = val;
		return true;
	}
	cerr << ThePrgName << ": string value expected for the `" << name << "` option; got `" << valStr(val) << "'" << endl;
	return false;
}

void StrOpt::report() const {
	Comment << valStr(theVal);
}


/* NetAddr */

NetAddrOpt::NetAddrOpt(OptGrp *aGrp, const char *aName, const char *aDescr, const NetAddr &def):
	Opt(aGrp, aName, aDescr), theVal(def) {
}

bool NetAddrOpt::parse(const char *name, const char *val) {
	String pref, suf;
	if (val && splitVal(val, pref, suf)) {
		int p;
		if (isInt(suf, p) && p >= 0) {
			theVal.host(pref);
			theVal.port(p);
			return true;
		} else
			cerr << ThePrgName << ": invalid port specification for the `" << name << "` option; got `" << valStr(val) << "'" << endl;
	} else
		cerr << ThePrgName << ": net address (<host>:<port>) value expected for the `" << name << "` option; got `" << valStr(val) << "'" << endl;
	return false;
}

void NetAddrOpt::report() const {
	Comment << theVal;
}


/* HelpOpt */

HelpOpt::HelpOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	Opt(aGrp, aName, aDescr) {
}

bool HelpOpt::parse(const char *, const char *) {
	Assert(theCmdLine);
	theCmdLine->usage(cerr);
	exit(0);
}


/* TwoInt */

TwoIntOpt::TwoIntOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	Opt(aGrp, aName, aDescr), theLoVal(-1), theHiVal(-2) {
}

bool TwoIntOpt::parse(const char *name, const char *val) {
	if (val) {
		String pref, suf;
		// XXX: "eph..." does not really belong to this level
		if (!strcmp(val, "none") || !strcmp(val, "ephemeral") || !strcmp(val, "eph")) {
			theLoVal = -1;
			theHiVal = -2;
			return true;
		} else
		if (splitVal(val, pref, suf)) {
			if (isInt(pref, theLoVal) && isInt(suf, theHiVal)) {
				if (theLoVal >= 0 && theHiVal >= 0 && theLoVal <= theHiVal)
					return true;
			}
		} 
	}
	cerr << ThePrgName << ": <low>:<hi> expected for the `" << name << "` option; got `" << valStr(val) << "'" << endl;
	return false;
}

void TwoIntOpt::report() const {
	if (set())
		Comment << theLoVal << ':' << theHiVal;
	else
		Comment << "<none>";
}
