
/*
 * $Id: polysrv.cc,v 1.1.1.1 1999/01/27 08:40:44 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <new.h>
#include <signal.h>

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>

#include "Clock.h"
#include "Server.h"
#include "ListenQueue.h"
#include "StatPhaseMgr.h"
#include "OLog.h"
#include "LogComment.h"
#include "SharedOpts.h"
#include "SrvOpts.h"

#include "gadgets.h"
#include "globals.h"
#include "logTags.h"

static ListenQueue TheLQueue;
static bool doShutdown = false;

// runs one select loop cycle
static
int step(Time *timeout) {
	FD_Set &rdSet = TheSelect->set(dirRead);
	FD_Set &wrSet = TheSelect->set(dirWrite);

	const int readyCount = TheSelect->select(timeout);
	CallStat(ThePinger, selected());

	if (readyCount > 0) {
		TheLQueue.begCycle(readyCount);
		for (int fd = TheSelect->maxFD(); fd >= 0; --fd) {
			if (rdSet.isReady(fd))
				rdSet[fd]->noteReadReady(fd);
			if (wrSet.isReady(fd))
				wrSet[fd]->noteWriteReady(fd);
			TheLQueue.tick(fd);
		}
		TheLQueue.endCycle();
	} else
	if (readyCount < 0)
		Must(errno == EINTR);

	return readyCount;
}

static
void shutDown(int sig) {
	Comment << "got shut down signal (" << sig << ")" << endc;
	doShutdown = true;
}

static
void configureLogs(int prec) {
	if (TheOpts.theConsFileName && strcmp(TheOpts.theConsFileName, "-"))
		cout = cerr = clog = *new ofstream(TheOpts.theConsFileName);

	if (TheOpts.theLogFileName)
		TheLog().stream(TheOpts.theLogFileName, new ofstream(TheOpts.theLogFileName));
	TheLog().resize(TheOpts.theLogSize);
	Comment.TheEchoLevel = TheOpts.theVerbLevel;

	configureStream(cout, prec);
	configureStream(cerr, prec);
	configureStream(clog, prec);
}

// set some general stuff and
// propogate cmd line options to corresponding objects
static
void configure() {

	configureLogs(2);

	// ignore some signals
	signal(SIGPIPE, SIG_IGN);
	signal(SIGHUP, SIG_IGN);

	// quit nicely on some signals
	signal(SIGINT, &shutDown);

	// set resource limits
	Check(SetMaxFD(Select::TheFDLimit) > 0);

	TheStatPhaseMgr.goal(TheOpts.theGoal);
}

int main(int argc, char *argv[]) {

	CmdLine cmd(Array<OptGrp*>() << &TheOpts << &TheSrvOpts);
	if (!cmd.parse(argc, argv) || !TheOpts.validate() || !TheSrvOpts.validate())
		return -1;

	Clock::Update(false);

	configure();

	cmd.report();

	const UniqId runId = UniqId::Create();
	Comment(1) << "run-id: " << runId << " pid: " << getpid() << endc;
	TheLog() << bege(lgRunId) << runId << ende;

	Clock::Update();
	if (TheOpts.theGoal < 0)
		Comment << "waiting for requests..." << endc;
	else
		Comment << "waiting for " << TheOpts.theGoal << " requests..." << endc;

	const Time start = TheClock;

	ThePinger.start();
	TheSoOftenPinger.start();

	Server TheServer;
	TheServer.configure(TheSrvOpts.thePort);
	TheServer.listenQueue(&TheLQueue);
	TheServer.start();

	Time *tout = TheSrvOpts.theTimeout >= 0 ? &TheSrvOpts.theTimeout : 0;
	while (!doShutdown && (TheOpts.theGoal < 0 || TheXactComplCnt < TheOpts.theGoal)) {
		Clock::Update();
		ThePinger.checkpoint();
		TheSoOftenPinger.checkpoint();
		if (!step(tout) && tout) {
			Comment << "shutting down after at least " << *tout << " of inactivity" << endc;
			doShutdown = true;
		}
	}

	Clock::Update();
	TheStatPhaseMgr.forceFlush();

	Comment << "sent " << TheXactComplCnt << " replies; roughly " 
		<< Ratio(TheXactComplCnt, (TheClock - start).secd()) << " rep/sec"
		<< endc;
	return 0;
}
